import os
from platform import system
from re import match, sub
from datetime import datetime
from loguru import logger
from getpass import getpass
from configparser import ConfigParser


class Settings:
    def __init__(self, check=False):
        self.ini_path = os.path.join(get_settings_path(), 'rsack_settings.ini')
        # If settings doesn't exist then create one
        if not os.path.isfile(self.ini_path):
            logger.debug(f'Generating {self.ini_path}')
            self.generate_settings()
        # Read settings.ini
        self.config = ConfigParser()
        self.config.read(self.ini_path)

    @logger.catch
    def generate_settings(self):
        """
        Generates settings.ini based on user input
        """
        # Bugs.co.kr Inputs
        bugs_username = input(
            'Please enter your Bugs.co.kr email: (If none leave blank)\n')
        bugs_password = getpass(
            'Please enter your Bugs.co.kr password: (If none leave blank)\n')
        bugs_threads = input('How many Bugs.co.kr download threads do you want to use?: (2-3 recommended)\n')
        bugs_path = input('Enter your download path for Bugs.co.kr:\n')
        bugs_lyrics = input('Enter your preferred Bugs.co.kr lyrics type: (T = Timed, N = Normal)\n')
        
        # Genie.co.kr Inputs
        genie_username = input(
            'Please enter your Genie.co.kr username: (If none leave blank)\n')
        genie_password = getpass(
            'Please enter your Genie.co.kr password: (If none leave blank)\n')
        genie_threads = input('How many Genie.co.kr download threads do you want to use?: (2-3 recommended)\n')
        genie_path = input('Enter your download path for Genie.co.kr:\n')
        # Write file
        config = ConfigParser()
        config['Bugs'] = {'username': bugs_username,
                          'password': bugs_password,
                          'threads': bugs_threads,
                          'path': bugs_path,
                          'lyrics': bugs_lyrics}
        config['Genie'] = {'username': genie_username,
                           'password': genie_password,
                           'threads': genie_threads,
                           'path': genie_path}
        with open(self.ini_path, 'w+') as configfile:
            config.write(configfile)

    def Bugs(self):
        """
        Returns the contents of the 'Bugs' section of settings.ini as dict.
        """
        return dict(self.config.items('Bugs'))

    def Genie(self):
        """
        Returns the contents of the 'Genie' section of settings.ini as dict.
        """
        return dict(self.config.items('Genie'))


def track_to_flac(track, album, lyrics):
    """Creates dict with appropriate FLAC headers.

    Args:
        track (dict): Dict containing track info from API response
        album (dict): Dict containing album info from API response
        lyrics (str): Lyrics of the track
    """
    meta = {
        "ALBUM": track['album_title'],
        "ALBUMARTIST": track['artist_disp_nm'],
        "ARTIST": track['artist_disp_nm'],
        "TITLE": track['track_title'],
        "DISCNUMBER": str(track['disc_id']),
        "DISCTOTAL": str(album['disc_total']),
        "TRACKNUMBER": str(track['track_no']),
        "TRACKTOTAL": str(track['track_total']),
        "COMMENT": str(track['track_id']),
        "DATE": _format_date(track['release_ymd']),
        "GENRE": album['Genre'],
        "LABEL": album['Label'],
        "LYRICS": lyrics
    }
    return meta


def _format_date(date):
    # Bugs sometimes does not include the day on the release date for older albums released on the first day of the month.
    # We will append it manuallly before date transformation.
    if len(date) == 6:
        date = date + "01"
    date_patterns = ["%Y%m%d", "%Y%m", "%Y"]
    for pattern in date_patterns:
        try:
            return datetime.strptime(date, pattern).strftime('%Y.%m.%d')
        except ValueError:
            pass


def determine_quality(svc_flac_yn):
    if svc_flac_yn == 'Y':
        return 'flac'
    else:
        return 'mp3'

def bugs_id(url):
	return match(
		r'https?://music\.bugs\.co\.kr/(?:(?:album|artist|track|playlist)/|[a-z]{2}-[a-z]{2}-?\w+(?:-\w+)*-?)(\w+)',
		url).group(1)

def genie_id(url):
	"""
	:param url: Genie url
	:return: Album ID
	"""
	expression = "https://genie.co.kr/detail/(artistInfo|albumInfo)......([0-9]*)"
	result = match(expression, url)
	if result:
		return result
	logger.critical("Invalid URL: {}".format(url))
 
def get_settings_path():
    if "XDG_CONFIG_HOME" in os.environ:
        return os.environ['XDG_CONFIG_HOME']
    elif "HOME" in os.environ:
        return os.environ['HOME']
    elif "HOMEDRIVE" in os.environ and "HOMEPATH" in os.environ:
        return os.environ['HOMEDRIVE'] + os.environ['HOMEPATH']
    else:
        return os.path._getfullpathname("./")

def get_ext(type):
    """Returns file extension

    Args:
        type (str): FILE_EXT from API response

    Returns:
        [str]: file extension string
    """
    if type == "MP3":
        return "mp3"
    else:
        return "flac"

def insert_total_tracks(tracks):
    """Add total_tracks to track metadata

    Args:
        tracks (list): list of dicts containing track metadata
    """
    total_tracks_by_disc_id = {}
    for track in tracks:
        if track["disc_id"] not in total_tracks_by_disc_id:
            total_tracks_by_disc_id[track["disc_id"]] = 0

        total_tracks_by_disc_id[track["disc_id"]] += 1

    for track in tracks:
        track["track_total"] = total_tracks_by_disc_id[track["disc_id"]]

def _is_win():
	if system() == 'Windows':
		return True

def sanitize(fn):
	"""
	:param fn: Filename
	:return: Sanitized string
	Removes invalid characters in the filename dependant on Operating System.
	"""
	if _is_win():
		return sub(r'[\/:*?"><|]', '_', fn)
	else:
		return sub('/', '_', fn)


import os
import mutagen.id3 as id3
from loguru import logger
from urllib.parse import unquote
from mutagen.flac import FLAC
from rsack.clients import genie
from rsack.utils import Settings, get_ext, sanitize
from concurrent.futures import ThreadPoolExecutor

class Download:
    def __init__(self, type, id):
        self.settings = Settings().Genie()
        self.client = genie.Client()
        self.client.auth(
            username=self.settings['username'], password=self.settings['password'])
        self.meta = self._collect(type, id)
        if type == "artist":
            self._artist(id)
        elif type == "album":
            self._album(id)

    def _artist(self, id):
        """Iterate albums in the artists album list

        Args:
            id (int): Artist UID

        Note:
            Artist album lists do not contain any track information so a POST to song/j_AlbumSongList.json is still necessary.
            Perhaps self.meta['artist_song_list'] could be utilized
        """
        for album in self.meta['DataSet']['DATA']:
            self._album(album['ALBUM_ID'])

    def _album(self, id):
        """Iterate tracks in the album list

        Args:
            id (int): Album UID
        """
        meta = self.client.get_album(id)
        logger.info(
            f"Album: {unquote(meta['DATA0']['DATA'][0]['ALBUM_NAME'])}")
        artist_name = sanitize(
            unquote(meta['DATA0']['DATA'][0]['ARTIST_NAME']))
        self.album_path = os.path.join(
            self.settings['path'], artist_name, f"{artist_name} - {sanitize(unquote(meta['DATA0']['DATA'][0]['ALBUM_NAME']))}")
        if not os.path.isdir(self.album_path):
            os.makedirs(self.album_path)
        self._download_cover(
            unquote(meta['DATA0']['DATA'][0]['ALBUM_IMG_PATH_600']))

        # Create dict containing relevant album information for tagging.
        self.f_meta = {
            "album_title": meta['DATA0']['DATA'][0]['ALBUM_NAME'],
            "track_total": len(meta['DATA1']['DATA']),
            "album_artist": unquote(meta['DATA0']['DATA'][0]['ARTIST_NAME']),
            "release_date": meta['DATA0']['DATA'][0]['ALBUM_RELEASE_DT'],
            "planning": unquote(meta['DATA0']['DATA'][0]['ALBUM_PLANNER'])
        }
        self.f_meta['disc_total'] = meta['DATA1']['DATA'][self.f_meta['track_total'] - 1]['ALBUM_CD_NO']

        tracks = []
        track_numbers = []
        disc_numbers = []
        track_artist = []
        for track in meta['DATA1']['DATA']:
            tracks.append(track['SONG_ID'])
            track_numbers.append(track['ALBUM_TRACK_NO'])
            disc_numbers.append(track['ALBUM_CD_NO'])
            track_artist.append(track['ARTIST_NAME'])

        logger.info(f"Threads: {self.settings['threads']}")
        with ThreadPoolExecutor(max_workers=int(self.settings['threads'])) as executor:
            executor.map(self._track, tracks, track_numbers,
                         disc_numbers, track_artist)

    def _track(self, id, track_number, disc_number, track_artist):
        meta = self.client.get_stream_meta(id, '24bit')
        logger.info(f"Track: {unquote(meta['SONG_NAME'])}")
        ext = get_ext(meta['FILE_EXT'])
        file_path = os.path.join(
            self.album_path, f"{track_number}. {sanitize(unquote(meta['SONG_NAME']))}.{ext}")
        r = self.client.session.get(unquote(meta['STREAMING_MP3_URL']))
        r.raise_for_status()
        with open(file_path, 'wb') as f:
            for chunk in r.iter_content(32 * 1024):
                if chunk:
                    f.write(chunk)
        self._fix_tags(file_path, ext, track_number, disc_number,
                       track_artist, unquote(meta['SONG_NAME']))

    def _fix_tags(self, path, ext, track_number, disc_number, track_artist, track_title):
        if ext == "mp3":
            try:
                audio = id3.ID3(path)
            except id3.ID3NoHeaderError:
                audio = id3.ID3()
            audio['TIT2'] = id3.TIT2(text=str(track_title))
            audio['TALB'] = id3.TALB(text=str(self.f_meta['album_title']))
            audio['TCON'] = id3.TCON(text=str(self.f_meta['album_title']))
            audio['TRCK'] = id3.TRCK(
                text=str(track_number) + "/" + str(self.f_meta['track_total']))
            audio['TPOS'] = id3.TPOS(
                text=str(disc_number) + "/" + str(self.f_meta['disc_total']))
            audio['TDRC'] = id3.TDRC(text=self.f_meta['release_date'])
            audio['TPUB'] = id3.TPUB(text=self.f_meta['planning'])
            audio['TPE1'] = id3.TPE1(text=unquote(track_artist))
            audio['TPE2'] = id3.TPE2(text=self.f_meta['album_artist'])
            audio.save(path, "v2_version=3")
        else:
            audio = FLAC(path)
            audio['TRACKTOTAL'] = str(self.f_meta['track_total'])
            audio['DISCTOTAL'] = str(self.f_meta['disc_total'])
            audio['DATE'] = self.f_meta['release_date']
            audio['LABEL'] = self.f_meta['planning']
            audio['ARTIST'] = unquote(track_artist)
            audio['ALBUMARTIST'] = self.f_meta['album_artist']
            audio.save()

    def _download_cover(self, url):
        path = os.path.join(self.album_path, 'cover.jpg')
        if not os.path.isfile(path):
            r = self.client.session.get(unquote(url))
            with open(path, 'wb') as f:
                f.write(r.content)
        else:
            logger.debug(f"{path} already exists.")

    def _collect(self, type, id):
        """Returns metadata for specified release

        Args:
            type (str): Release type (album/artist)
            id (int): Artist or Album UID

        Returns:
            [dict]: API response
        """
        if type == 'artist':
            return self.client.get_artist(id)
        if type == 'album':
            return self.client.get_album(id)



import requests
from loguru import logger

class Client:

    def __init__(self):
        self.session = requests.Session()
        self.dev_id = "eb9d53a3c424f961"

        self.session.headers.update({
            "User-Agent": "genie/ANDROID/5.1.1/WIFI/SM-G930L/dreamqltecaneb9d53a3c424f961/500200714/40807",
            "Referer": "app.genie.co.kr"
        })

    def make_call(self, sub, epoint, data):
        """
        :param sub: Url Prefix
        :param epoint: Endpoint
        :param data: Post data
        :return: API Response
        Endpoints used:
                player/j_StmInfo.json - Provides information on the streamed track.
                member/j_Member_Login.json - Authentication.
                song/j_AlbumSongList.json - Provides album information.
        """
        r = self.session.post(
            "https://{}.genie.co.kr/{}".format(sub, epoint), data=data)
        r.raise_for_status()

        return r.json()

    def auth(self, username, password):
        """
        Authenticate our session appearing as an Android device
        """
        data = {
            "uxd": username,
            "uxx": password
        }
        r = self.make_call("app", "member/j_Member_Login.json", data)
        if r['Result']['RetCode'] != "0":
            logger.critical("Authentication failed.")
        else:
            logger.info("Login Successful.")
        self.usr_num = r['DATA0']['MemUno']
        self.usr_token = r['DATA0']['MemToken']
        self.stm_token = r['DATA0']['STM_TOKEN']

    def get_album(self, id):
        """
        :param id: Album ID.
        :return: API Response containing album metadata.
        """
        data = {
            "axnm": id,
            "dcd": self.dev_id,
            "mts": "Y",
            "stk": self.stm_token,
            "svc": "IV",
            "tct": "Android",
            "unm": self.usr_num,
            "uxtk": self.usr_token
        }
        r = self.make_call("app", "song/j_AlbumSongList.json", data)
        if r['Result']['RetCode'] != "0":
            logger.critical("Failed to retrieve metadata")
        return r
    
    def get_artist(self, id):
        """Returns artist information

        Args:
            id (int): Artist UID

        Returns:
            [dict]: API Response containing artist information
        """
        data = {
            "uxtk": self.usr_token,
            "sign": "Y",
            "tct": "Android",
            "svc": "IV",
            "stk": self.stm_token,
            "dcd": self.dev_id,
            "xxnm": id,
            "unm": self.usr_num,
            "mts": "Y"
        }
        r = self.make_call("app", "song/j_ArtistAlbumList.json", data)
        if r['Result']['RetCode'] != "0":
            logger.critical("Failed to retrieve metadata")
        return r

    def get_stream_meta(self, id, q):
        """
        :param id: Album ID
        :param q: Album quality.
        :return: API Response containing metadata for the currently streamed track.
        Quality options:
                1 - MP3
                2 - 16bit FLAC
                3 - 24bit FLAC
        """
        data = {
            "bitrate": q,
            "dcd": self.dev_id,
            "stk": self.stm_token,
            "svc": "IV",
            "unm": self.usr_num,
            "uxtk": self.usr_token,
            "xgnm": id
        }
        r = self.make_call("stm", "player/j_StmInfo.json", data)
        if r['Result']['RetCode'] == "A00003":
            logger.critical("Device ID has changed since last stream resulting in playback error.")
        if r['Result']['RetCode'] != "0":
            logger.critical("Failed to retrieve metadata")
        if r['Result']['RetCode'] == "S00001":
            logger.debug("This content is currently unavailable for service")
        return r['DataSet']['DATA'][0]



import os
import requests
from datetime import datetime
from loguru import logger
from mutagen.flac import FLAC, Picture
import mutagen.id3 as id3
from mutagen.id3 import ID3NoHeaderError
from concurrent.futures import ThreadPoolExecutor

from rsack.clients import bugs
from rsack.utils import Settings, track_to_flac, determine_quality, insert_total_tracks


class Download:

    def __init__(self, type, id):
        """Initialize download

        Args:
            type (str): Release type (album/artist/track)
            id (int/str): UID of album/artist/track
        """
        # Initialize the type and ID of the release
        self.type = type
        self.id = id
        self.settings = Settings().Bugs()

        # Create and authorize the client
        self.client = bugs.Client(self.settings)
        self.conn_info = self.client.auth()
        self.api_key = self.client.get_api_key()

        # Grab the metadata
        self.meta = self.collect()
        # Acquire disc total by finding the last track entry and tacking the disc number
        self.meta['disc_total'] = self.meta['Tracks'][-1]['disc_id']
        # Add track_total to meta.
        insert_total_tracks(self.meta['Tracks'])

        # Construct album path
        self.album_path = os.path.join(
            self.settings['path'], self.meta['Album_artist'], f"{self.meta['Album_artist']} - {self.meta['Album']}")
        if not os.path.exists(self.album_path):
            logger.debug(f"Creating {self.album_path}")
            os.makedirs(self.album_path)
        else:
            pass

        # Download album cover
        self.cover_path = self._download_cover(self.album_path)

        # Begin downloading tracks
        logger.info(f"Threads: {self.settings['threads']}")
        with ThreadPoolExecutor(max_workers=int(self.settings['threads'])) as executor:
            executor.map(self._download, self.meta['Tracks'])

    def _download(self, track):
        """Downloads the track and passes it on for tagging

        Args:
            track ([dict]): Track metadata provided in API response
        """
        track_id = track['track_id']
        logger.debug(f"Processing: {track_id}")
        # Create params required to request the track
        params = {
            "ConnectionInfo": self.conn_info,
            "api_key": self.api_key,
            "overwrite_session": "Y",
            "track_id": track_id
        }
        # Request track
        r = requests.get(
            "http://api.bugs.co.kr/3/tracks/{}/listen/android/flac".format(
                track_id),
            params=params, stream=True)
        if r.status_code == 404:
            logger.info(f"{track['track_title']} unavailable, skipping.")
        else:
            # Directory management
            file_path = os.path.join(
                self.album_path, f"{track['track_no']}. {track['track_title']}.{determine_quality(track['svc_flac_yn'])}")
            with open(file_path, 'wb') as f:
                for chunk in r.iter_content(32 * 1024):
                    if chunk:
                        f.write(chunk)
            self._tag(track, file_path)
            logger.info(f"{track['track_title']} downloaded and tagged")

    def _download_cover(self, path):
        """Downloads album artwork

        Args:
            path ([path]): Path of folder to download to

        Returns:
            [path]: Path of .jpg file
        """
        cover_path = os.path.join(path, 'cover.jpg')
        if os.path.exists(cover_path):
            logger.info('Cover already exists, skipping.')
        else:
            r = requests.get(self.meta['Cover_url']['500'])
            r.raise_for_status
            with open(cover_path, 'wb') as f:
                f.write(r.content)
            logger.info('Cover artwork downloaded.')
        return cover_path

    def _tag(self, track, file_path):
        """Applies appropriate tags to the audio file

        Args:
            track ([dict]): Dict containing track information
            file_path ([path]): Path of .flac or .mp3 file
        """
        lyrics = self._get_lyrics(track['track_id'], track['lyrics_tp'])
        tags = track_to_flac(track, self.meta, lyrics)
        # If file is FLAC
        if str(file_path).endswith('.flac'):
            f_file = FLAC(file_path)
            # Add cover artwork to flac file
            if self.cover_path:
                f_image = Picture()
                f_image.type = 3
                f_image.desc = 'Front Cover'
                with open(self.cover_path, 'rb') as f:
                    f_image.data = f.read()
                f_file.add_picture(f_image)
            logger.debug(f"Writing tags to {file_path}")
            for k, v in tags.items():
                f_file[k] = str(v)
            f_file.save()
        # If file is MP3
        if str(file_path).endswith('.mp3'):
            # Legend contains all ID3 tags for each FLAC header.
            legend = {
                "ALBUM": id3.TALB,
                "ALBUMARTIST": id3.TPE2,
                "ARTIST": id3.TPE1,
                "COMMENT": id3.COMM,
                "COMPOSER": id3.TCOM,
                "COPYRIGHT": id3.TCOP,
                "DATE": id3.TDRC,
                "GENRE": id3.TCON,
                "ISRC": id3.TSRC,
                "LABEL": id3.TPUB,
                "PERFORMER": id3.TOPE,
                "TITLE": id3.TIT2,
                "LYRICS": id3.USLT
            }
            try:
                m_file = id3.ID3(file_path)
            except ID3NoHeaderError:
                m_file = id3.ID3()
            logger.debug(f"Writing tags to {file_path}")
            # Apply tags using the legend
            for k, v in tags.items():
                try:
                    id3tag = legend[k]
                    m_file[id3tag.__name__] = id3tag(encoding=3, text=v)
                except KeyError:
                    continue
            # Track and disc numbers
            m_file.add(
                id3.TRCK(encoding=3, text=f"{track['track_no']}/{track['track_total']}"))
            m_file.add(
                id3.TPOS(encoding=3, text=f"{track['disc_id']}/{self.meta['disc_total']}"))
            # Apply cover artwork
            if self.cover_path:
                with open(self.cover_path, 'rb') as cov_obj:
                    m_file.add(id3.APIC(3, 'image/jpg', 3, '', cov_obj.read()))
            m_file.save(file_path, 'v2_version=3')

    def _get_lyrics(self, track_id, lyrics_tp):
        """Retrieves and formats track lyrics

        Args:
            track_id ([int/str]): UID of the track
            lyrics_tp ([str]): 'T' or 'N' is provided in the API reponse, pass this. (T = Timed, N = Normal) 

        Returns:
            [str]: Formatted lyrics
        """
        # If user prefers timed then retrieve timed lyrics
        if lyrics_tp and self.settings['lyrics'] == 'T':
            # Retrieve timed lyrics
            r = requests.get(
                f"https://music.bugs.co.kr/player/lyrics/T/{str(track_id)}")
            # Format timed lyrics
            lyrics = r.json()['lyrics'].replace("＃", "\n")
            line_split = (line.split('|') for line in lyrics.splitlines())
            lyrics = ("\n".join(
                f'[{datetime.fromtimestamp(round(float(a), 2)).strftime("%M:%S.%f")[0:-4]}]{b}' for a, b in
                line_split))
        # If user prefers untimed or the only available is untimed then use untimed
        if 'lyrics_tp' or self.settings['lyrics'] == "N":
            r = requests.get(
                'https://music.bugs.co.kr/player/lyrics/N/{}'.format(str(track_id)))
            lyrics = r.json()['lyrics']
            # If unavailable leave as empty string
            if lyrics_tp is None:
                lyrics = ""
        return lyrics

    def collect(self):
        """Returns specified metadata from Bugs.co.kr API

        Args:
            type (string): specify the release type, album/artist/track
            id (int/string): unique identifier of the album, artist or track.

        Returns:
            [dict]: dict containing usable metadata for tagging
        """
        meta = self.client.get_meta(type=self.type, id=int(self.id))
        return self._clean(meta)

    def _clean(self, meta):
        """Cuts out all unnecessary data and returns

        Args:
            meta ([dict]): Metadata provided in api response

        Returns:
            [dict]: Cleaned metadata
        """
        clean_meta = {"Album": meta['list'][0]['album_info']['result']['title'],
                      "Album_artist": meta['list'][0]['album_info']['result']['artist_disp_nm'],
                      "Genre": meta['list'][0]['album_info']['result']['genre_str'].replace(",", "; "),
                      "Label": '; '.join(str(label['label_nm']) for label in meta['list'][0]['album_info']['result']['labels']),
                      "Cover_url": meta['list'][0]['album_info']['result']['img_urls'],
                      "Tracks": meta['list'][0]['album_info']['result']['tracks'],
                      }
        return clean_meta



# Standard
import requests
from loguru import logger

class Client:
    def __init__(self, settings):
        self.session = requests.Session()
        self.settings = settings
        self.api_key = "b2de0fbe3380408bace96a5d1a76f800"
        self.session.headers.update({
            "User-Agent": "Mobile|Bugs|4.11.30|Android|5.1.1|SM-G965N|samsung|market",
            "Host": "api.bugs.co.kr",
        })
 
    def auth(self):
        data = {
            "device_id": "gwAHWlkOYX_T8Sl43N78GiaD6Sg_",
            "passwd": self.settings['password'],
            "userid": self.settings['username']
        }
        r = self.make_call("secure", "mbugs/3/login?", data=data)
        if r['ret_code'] == 300:
            logger.critical("Authentication Error, Invalid Credentials")
        else:
            logger.info("Login Successful")
        self.nickname = r['result']['extra_data']['nickname']
        self.connection_info = r['result']['coninfo']
        return self.connection_info

    def get_api_key(self):
        return self.api_key

    def make_call(self, sub, epoint, data=None, json=None, params=None):
        r = self.session.post("https://{}.bugs.co.kr/{}api_key={}".format(sub, epoint, self.api_key), json=json, data=data, params=params)
        return r.json()

    @logger.catch
    def get_meta(self, type, id):
        if type == "album":
            json=[{"id":"album_info","args":{"albumId":id}}, {"id":"artist_role_info","args":{"contentsId":id,"type":"ALBUM"}}]
        elif type == "artist":
            json=[{"id":"artist_info","args":{"artistId":id}}, {"id":"artist_album","args":{"artistId":id, "albumType":"main","tracksYn":"Y","page":1,"size":500}}]
        elif type == "track":
            json=[{"id":"track_detail", "args":{"trackId":id}}]
        else:
            logger.critical("Invalid invokeMap type.")
        r = self.make_call("api", "3/home/invokeMap?", json=json)
        if r['ret_code'] != 0:
            raise logger.critical("Failed to get a map.")
        return r


from loguru import logger
from argparse import ArgumentParser

from rsack import bugs, genie
from rsack.utils import bugs_id, genie_id, Settings

def get_args():
    parser = ArgumentParser()
    parser.add_argument('-b', '--bugs', action='store_true', dest="bugs", required=False)
    parser.add_argument('-g', '--genie', action='store_true', dest="genie", required=False)
    parser.add_argument('-u', '--url', nargs='*', dest="url", required=False)
    parser.add_argument('-cfg', '--config', action='store_true', dest="config", required=False)
    parser.add_argument('-o', '--open', action='store_true', dest="open", required=False)
    return parser.parse_args()

    
def main():
    args = get_args()
    if args.config:
            Settings(check=True)
    if args.bugs:
        for url in args.url:
            id = bugs_id(url)
            if "album" in url:
                bugs.Download(type="album", id=id)
            elif "artist" in url:
                logger.critical("Artist batching not yet available")
            elif "track" in url:
                bugs.Download(type="Single tracks not yet available", id=id)
    elif args.genie:
        for url in args.url:
            match = genie_id(url)
            if match.group(1) == "artistInfo":
                type = "artist"
            elif match.group(1) == "albumInfo":
                type = "album"
            else: # Catch invalid info types
                logger.critical("URL type unkown")
            genie.Download(type=type, id=match.group(2))
            
